//****************************************************************************
//	
//	Copyright 2020 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : i440GetSamples.cpp
//
//	Project	 : I440 Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	01/24/20	  1.1		Original Release
//
//****************************************************************************
//
// Test for functions:
//   int InitializeSession(void)
//   int SetChannelMode(unsigned int channel, int duplex, int range)
//   int SetTimeBase(unsigned int time)
//   int GetChannelDataCount(unsigned char *valBuf, unsigned int channel, unsigned int *count)
//   int ConvertToVolts(float *voltage, unsigned int channel, int value)
//   int CloseSession(void)
//
//****************************************************************************

#include <stdio.h>
#include <stdlib.h>  
#include <tchar.h>  
#include <conio.h>  
#include <windows.h>
#include "i440DLL.h"

#define APP_NAME        "i440GetSamples"
#define MAJOR_VER       1
#define MINOR_VER       0
#define I440_PERF       0
#define DATA_DISPLAY    0

void print_usage_and_exit(void)
{
    printf("Usage error:\n");
    printf("  i440GetSamples <Channel> <Count> <Timebase>\n");
    printf("  Channel must be between 0 and 7\n");
    printf("  Count can be any number less than 65536\n");
    printf("  Timebase can be any number greater than 58\n");
    printf("  Timebase is optional, default value is 59\n");
    exit(1);
}

int _tmain(int argc, _TCHAR* argv[])
{
    unsigned int ch = 0;
    unsigned int cnt = 10;
    unsigned int time = TIME_1MHz;
    unsigned int totalBytes = 0;
    unsigned char *Buffer;
    unsigned int data;
    float convVoltage;
    int dllReturn = SUCCESS;

#if I440_PERF == 1
    INT64 ctr1 = 0, ctr2 = 0, freq = 0;
#endif

    printf("I440 Application : %s\n", APP_NAME);
    printf("Version %d.%d\n", MAJOR_VER, MINOR_VER);

    if (argc == 3 || argc == 4)
    {
        // load channel
        ch = _ttoi(argv[1]);

        // load count
        cnt = _ttoi(argv[2]); 

        // load time base if included
        if (argc == 4)
            time = _ttoi(argv[3]);

        printf("\n%d samples on channel %d at timebase %d\n", cnt, ch, time);

        // total memory size required
        totalBytes = cnt * 3;
    }
    else
        print_usage_and_exit();

    dllReturn = InitializeSession();

    if (dllReturn)
    {
        printf("\nError initializing device!\n");
        exit(dllReturn);
    }
    else   
    {
        printf("\nFT232 Sync FIFO Mode opened.\n");
    }

    // configure channel
    SetChannelMode(ch, ADC_BIPOLAR, ADC_TOP_10_24V);

    // set time base
    dllReturn = SetTimeBase(time);

    if (dllReturn)
    {
        printf("\nError setting time base!\n");
        goto Exit;
    }

    // allocate memory for samples
    Buffer = (unsigned char *) malloc(totalBytes);

    if (Buffer == NULL)
    {
        printf("\nInsufficient memory available!\n");
        goto Exit;
    }
    else
        printf("\n%d bytes of memory space allocated for sample storage.\n\n", totalBytes);

#if I440_PERF == 1
    QueryPerformanceFrequency((LARGE_INTEGER *)&freq);
    QueryPerformanceCounter((LARGE_INTEGER *)&ctr1);
#endif

    // get samples
    printf("\nGet %d samples for channel %d\n", cnt, ch);
    dllReturn = GetChannelDataCount(Buffer, ch, &cnt);

    if (dllReturn)
    {
        printf("\nError %d executing conversions for channel %d!\n", dllReturn, ch);
        FlushRxQueue();
    }
    else
    {
#if I440_PERF == 1
        QueryPerformanceCounter((LARGE_INTEGER *)&ctr2);
        printf("%d samples in %.6f secs\n", cnt, ((ctr2 - ctr1) * 1.0 / freq));
#endif

        // display samples as voltages
        for (unsigned int i = 0; i < cnt * 3; i += 3)
        {
            ch = (*(Buffer + i + 2) >> 3) & 0x7;
            data = (*(Buffer + i) << 8) | *(Buffer + i + 1);

            ConvertToVolts(&convVoltage, ch, data);

            printf("Sample %5d - Channel %d - Voltage = %.3f\n", i / 3 + 1, ch, convVoltage);
        }
    }

    // free memory
    free(Buffer);

Exit:
    // close device
    dllReturn = CloseSession();

    if (dllReturn)
    {
        printf("\nError closing device!\n");
        exit(dllReturn);
    }
    else
    {
        printf("\nFT232 device closed.\n");
    }

    return(0);
}
