//****************************************************************************
//	
//	Copyright 2020 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : i440GetSequenceCount.cpp
//
//	Project	 : I440 Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	01/24/20	  1.0		Original Release
//
//****************************************************************************
//
// Test for functions:
//   int InitializeSession(void)
//   int SetChannelMode(unsigned int channel, int duplex, int range)
//   int SetTimeBase(unsigned int time)
//   int GetChannelSequenceCount(unsigned char *valBuf, unsigned int *chanSeq, unsigned int count)
//   int ConvertToVolts(float *voltage, unsigned int channel, int value)
//   int CloseSession(void)
//
//****************************************************************************

#include <stdio.h>
#include <stdlib.h>  
#include <tchar.h>  
#include <windows.h>
#include "i440DLL.h"

#define APP_NAME    "i440GetSequenceCount"
#define MAJOR_VER   1
#define MINOR_VER   0

void print_usage_and_exit(void)
{
    printf("Usage error:\n");
    printf("  i440GetSequenceCount <Count> <Timebase>\n");
    printf("  Count can be any number less than 65536\n");
    printf("  Count is optional, default value is 11\n");
    printf("  Timebase can be any number greater than 58\n");
    printf("  Timebase is optional, default value is 59\n");
    exit(1);
}

int _tmain(int argc, _TCHAR* argv[])
{
    unsigned int chSeq[] = { 1, 3, 5, 7, 0xff };
    unsigned int size = sizeof(chSeq) / sizeof(unsigned int) - 1;
    unsigned int cnt = 12;
    unsigned int time = TIME_1MHz;
    unsigned int totalBytes = 0;
    unsigned int ch;
    unsigned int data;
    unsigned char *Buffer;
    float convVoltage;
    int dllReturn = SUCCESS;

    printf("I440 Application : %s\n", APP_NAME);
    printf("Version %d.%d\n", MAJOR_VER, MINOR_VER);

    if (argc == 2 || argc == 3)
    {
        // load count, add one for dummy conversion at start
        cnt = _ttoi(argv[1]);

        // load time base if included
        if (argc == 3)
            time = _ttoi(argv[2]);

        printf("\n%d samples of sequence at timebase %d\n", cnt, time);
    }
    else if (argc != 1)
        print_usage_and_exit();

    // total memory size required
    totalBytes = cnt * 3;

    dllReturn = InitializeSession();

    if (dllReturn)
    {
        printf("\nError initializing device!\n");
        exit(dllReturn);
    }
    else   
    {
        printf("\nFT232 Sync FIFO Mode opened.\n");
    }

    // configure all channels
    for (int i = 0; i < 8; i++)
    {
        dllReturn = SetChannelMode(i, ADC_BIPOLAR, ADC_TOP_10_24V);
        
        if (dllReturn)
        {
            printf("\nError configuring channel %d!\n", i);
            goto Exit;
        }
    }

    // set time base
    dllReturn = SetTimeBase(time);

    if (dllReturn)
    {
        printf("\nError setting time base!\n");
        goto Exit;
    }
    else
    {
        printf("\nTime base set to %d.\n", time);
    }

    // allocate memory for samples
    Buffer = (unsigned char *) malloc(totalBytes);

    if (Buffer == NULL)
    {
        printf("\nInsufficient memory available!\n");
        goto Exit;
    }
    else
        printf("\n%d bytes of memory space allocated for sample storage.\n\n", totalBytes);

    // get samples
    dllReturn = GetChannelSequenceCount(Buffer, chSeq, &cnt);

    if (dllReturn)
    {
        printf("\nError %d executing sequence!\n", dllReturn);
    }
    else
    {
        // display samples as voltages
        for (unsigned int i = 0; i < totalBytes; i += 3)
        {
            ch = (*(Buffer + i + 2) >> 3) & 0x7;
            data = (*(Buffer + i) << 8) | *(Buffer + i + 1);

            ConvertToVolts(&convVoltage, ch, data);

            printf("Sample %5d - Channel %d - Voltage = %.3f\n", i / 3 + 1, ch, convVoltage);
        }
    }

    // free memory
    free(Buffer);

Exit:
    // close device
    dllReturn = CloseSession();

    if (dllReturn)
    {
        printf("\nError closing device!\n");
        exit(dllReturn);
    }
    else
    {
        printf("\nFT232 device closed.\n");
    }

    return(0);
}
